package com.gloo.sokoban.game;

import com.gloo.sokoban.grid.Cell;
import com.gloo.sokoban.grid.Grid;
import com.gloo.sokoban.xsb.File;

public class Player {
    private int row;
    private int col;
    private Grid grid;

    public Player(File file) {
        Grid grid = file.getGrid();
        int playerRow = grid.getPlayerRow();
        int playerCol = grid.getPlayerCol();
        if (playerRow == -1 || playerCol == -1) {
            playerRow = 0;
            playerCol = 0;
        }
        this.row = playerRow;
        this.col = playerCol;
        assert isInsideGrid(row, col);
        this.grid = grid;
        getCell().plusPlayer();
    }

    public char[][] toCharArray() {
        return grid.toCharArray(); // Retourne la grille sur laquelle le joueur agit
    }

    public Cell getCell() {
        return this.grid.cells[this.row][this.col];
    }

    public void play(Play play) {
        int oldRow = this.row;
        int oldCol = this.col;
        int newRow = newRowNaive(this.row, play);
        int newCol = newColNaive(this.col, play);

        if (!isInsideGrid(newRow, newCol)) {
            return;
        }

        if (this.grid.cells[newRow][newCol].hasBox() && canMoveBox(newRow, newCol, play)) {
            moveBox(newRow, newCol, play);
        }

        if (this.grid.cells[newRow][newCol].canBeVisited()) {
            this.row = newRow;
            this.col = newCol;
        }

        this.grid.cells[oldRow][oldCol].minusPlayer();
        getCell().plusPlayer();
    }

    private boolean canMoveBox(int newRow, int newCol, Play play) {
        int newBoxRow = newRowNaive(newRow, play);
        int newBoxCol = newColNaive(newCol, play);
        return isInsideGrid(newBoxRow, newBoxCol) && this.grid.cells[newBoxRow][newBoxCol].canBeVisited();
    }

    private void moveBox(int newRow, int newCol, Play play) {
        int newBoxRow = newRowNaive(newRow, play);
        int newBoxCol = newColNaive(newCol, play);
        this.grid.cells[newRow][newCol].minusBox();
        this.grid.cells[newBoxRow][newBoxCol].plusBox();
    }

    private boolean isInsideGrid(int row, int col) {
        return row >= 0 && row < this.grid.getRows() && col >= 0 && col < this.grid.getCols();
    }

    private static int newRowNaive(int row, Play play) {
        switch (play) {
            case UP:
                return row - 1;
            case DOWN:
                return row + 1;
            default:
                return row;
        }
    }

    private static int newColNaive(int col, Play play) {
        switch (play) {
            case LEFT:
                return col - 1;
            case RIGHT:
                return col + 1;
            default:
                return col;
        }
    }

    public boolean isGameOver() {
        return this.grid.isGameOver();
    }

    public void print() {
        this.grid.print();
    }
}
