package com.gloo.sokoban.gui;
import java.awt.*;
import javax.swing.*;

import com.gloo.sokoban.game.Game;
import com.gloo.sokoban.game.Play;

public class GamePanel extends JPanel {
    private final Game game;
    private final int cellSize = 50;

    public GamePanel(String levelPath) {
        this.game = new Game(levelPath);

        setFocusable(true);
        addKeyListener(new java.awt.event.KeyAdapter() {
            @Override
            public void keyPressed(java.awt.event.KeyEvent e) {
                handleKeyPress(e.getKeyCode());
            }
        });
    }

    private void handleKeyPress(int keyCode) {
        Play play = Play.NONE;
        switch (keyCode) {
            case java.awt.event.KeyEvent.VK_UP -> play = Play.UP;
            case java.awt.event.KeyEvent.VK_DOWN -> play = Play.DOWN;
            case java.awt.event.KeyEvent.VK_LEFT -> play = Play.LEFT;
            case java.awt.event.KeyEvent.VK_RIGHT -> play = Play.RIGHT;
            case java.awt.event.KeyEvent.VK_Q -> {
                JOptionPane.showMessageDialog(this, "Au revoir !");
                System.exit(0);
            }
            case java.awt.event.KeyEvent.VK_H -> {
                JOptionPane.showMessageDialog(this, "Utilisez les flèches directionnelles pour déplacer le joueur.\nQ pour quitter.");
                return;
            }
        }

        if (play != Play.NONE) {
            game.play(play); // Effectuer le mouvement
            repaint(); // Mettre à jour l'affichage

            if (game.isOver()) {
                JOptionPane.showMessageDialog(this, "Félicitations ! Vous avez gagné !");
            }
        }
    }

    @Override
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        char[][] level = game.getLevel(); // Récupérer l'état actuel du niveau
        for (int row = 0; row < level.length; row++) {
            for (int col = 0; col < level[row].length; col++) {
                char cell = level[row][col];
                switch (cell) {
                    case '#' -> g.setColor(Color.DARK_GRAY); // Mur
                    case '$' -> g.setColor(new Color(139, 69, 19));    // Caisse
                    case '*' -> g.setColor(new Color(139, 69, 19));    // Caisse sur la destination
                    case '@' -> g.setColor(Color.BLUE);       // Joueur
                    case '+' -> g.setColor(Color.BLUE);       // Joueur sur la destination
                    default -> g.setColor(Color.WHITE);      // Case vide
                }
                g.fillRect(col * cellSize, row * cellSize, cellSize, cellSize);

                if (cell == '.' || cell == '*' || cell == '+') {
                    int pointSize = cellSize / 5; // Taille du point noir
                    int xCenter = col * cellSize + (cellSize - pointSize) / 2;
                    int yCenter = row * cellSize + (cellSize - pointSize) / 2;
                    g.setColor(Color.BLACK);
                    g.fillOval(xCenter, yCenter, pointSize, pointSize);
                }
                g.setColor(Color.BLACK);
                g.drawRect(col * cellSize, row * cellSize, cellSize, cellSize); // Bordure
            }
        }
    }

    @Override
    public Dimension getPreferredSize() {
        char[][] level = game.getLevel();
        return new Dimension(level[0].length * cellSize, level.length * cellSize);
    }
}
