package com.gloo.sokoban.tui;

import java.util.Scanner;

import com.gloo.sokoban.game.Game;
import com.gloo.sokoban.game.Play;

public class Main {
    public static void main(String[] args) {
        Scanner scanner = new Scanner(System.in);
        if (args.length == 0) {
            args = new String[] { "xsb/b.xsb" };
        }
        for (String path : args) {
            Game game = new Game(path);

            while (!game.isOver()) {
                clearScreen();
                game.print();
                if (!scanner.hasNextLine())
                    break;
                Play play = getPlay(scanner.nextLine());
                if (play == Play.QUIT)
                    break;
                if (play == Play.HELP) {
                    clearScreen();
                    System.out.println("Use 'u', 'd', 'l', 'r' to move the player up, down, left or right.");
                    System.out.println("Use 'q' to quit the game and 'h' to display this message.");
                    System.out.println("Press Enter to continue...");
                    scanner.nextLine();
                    continue;
                }
                game.play(play);
            }

            clearScreen();
            game.print();
            if (game.isOver()) {
                System.out.println("\nCongratulations! You won!");
            } else {
                System.out.println("\nGoodbye!");
            }
        }
        scanner.close();
    }

    private static void clearScreen() {
        System.out.print("\033[H\033[2J");
        System.out.flush();
    }

    private static Play getPlay(String move) {
        switch (move) {
        case "u":
            return Play.UP;
        case "d":
            return Play.DOWN;
        case "l":
            return Play.LEFT;
        case "r":
            return Play.RIGHT;
        case "q":
            return Play.QUIT;
        case "h":
            return Play.HELP;
        default:
            return Play.NONE;
        }
    }
}
