package com.gloo.sokoban.xsb;

import java.util.concurrent.atomic.AtomicInteger;

import com.gloo.sokoban.grid.Cell;
import com.gloo.sokoban.grid.CellType;
import com.gloo.sokoban.grid.Grid;

public class Parser {
    private String content; // This is the content of the file that we want to parse

    public Parser(String content) {
        this.content = content;
    }

    public Grid parse() {
        Grid grid = new Grid(getRows(), getCols());
        final AtomicInteger row = new AtomicInteger(0);
        content.lines().forEach(line -> {
            for (int col = 0; col < line.length(); col++) {
                switch (line.charAt(col)) {
                case ' ':
                    grid.cells[row.get()][col] = new Cell(CellType.EMPTY);
                    break;
                case '@':
                    grid.cells[row.get()][col] = new Cell(CellType.PLAYER);
                    break;
                case '#':
                    grid.cells[row.get()][col] = new Cell(CellType.WALL);
                    break;
                case '.':
                    grid.cells[row.get()][col] = new Cell(CellType.TARGET);
                    break;
                case '$':
                    grid.cells[row.get()][col] = new Cell(CellType.BOX);
                    break;
                case '+':
                    grid.cells[row.get()][col] = new Cell(CellType.PLAYER_ON_TARGET);
                    break;
                case '*':
                    grid.cells[row.get()][col] = new Cell(CellType.BOX_ON_TARGET);
                    break;
                default:
                    grid.cells[row.get()][col] = new Cell(CellType.EMPTY);
                    break;
                }
            }
            row.incrementAndGet();
        });

        for (int i = 0; i < grid.getRows(); i++) {
            for (int j = 0; j < grid.getCols(); j++) {
                if (grid.cells[i][j] == null) {
                    grid.cells[i][j] = new Cell(CellType.EMPTY);
                }
            }
        }
        return grid;
    }

    private int getRows() {
        return (int) content.lines().count();
    }

    private int getCols() {
        return content.lines().mapToInt(String::length).max().orElse(0);
    }
}
